const XLSX = require('xlsx');
const program = require('commander');
const fs = require('fs-extra');
const path = require('path');
const Preview = require("ro-crate").Preview;
const HtmlFile = require("ro-crate").HtmlFile;
const ROCrate = require("ro-crate").ROCrate;
const utf8 = require("utf8");

program
  .version("0.1.0")
  .description(
    "Extracts data from a spreadsheet to make an RO crate"
  )
  .arguments("<f>")
  .option("-c,  --cratescript [cratesript]", "URL of Crate-script directory")
  .option("-a, --additional-context [context]", "JSON file with context object")
  .option("-p, --places [places]", "Spreadsheet of places (courts)")
  .option("-d, --data [data]", "Data directory from which to copy image files")
  .action((f) => {file = f})
program.parse(process.argv);
const cratescript = program.cratescript;


const places = {};


function loadPlaces(file){
  var workbook = XLSX.readFile(file);
   const placeData = XLSX.utils.sheet_to_json(workbook.Sheets["Sheet1"]);
    //console.log(data);    
    for (let row of placeData) {
        //console.log(Object.keys(row));
        // Latitude	Longitude	Start date	End date	Label	Description
        
        var geo = {
            "@type": "Geocoordinates",
            "name": `Latitude: ${row["Latitude"]} Longitude: ${row["Longitude"]}`,
            "@id": `#${row["Latitude"]},${row["Longitude"]}`,
            "latitude": row["Latitude"],
            "longitude": row["Longitude"]   
        }

        place = {
            "@type": "Place",
            "@id": `#place_${row["Latitude"]},${row["Longitude"]}`,
            "geo": {"@id": geo["@id"]}
        }

        
        // 
        var place = {
            "@id": `#place_${row["Place"]}`,
            "@type": ["Place"],
            "name": row["Place"],
            "geo": {"@id": geo["@id"]},
            "startDate": row["Start date"],
            "endDate": row["End date"],
            "description": row["Description"],
            "note": row["note"]
        }

        crate.getGraph().push(place);
        crate.getGraph().push(geo);
        

        // Place logic
        places[place.name] = place;

        //crate.getGraph().push(geo); 
        console.log("Pushed", place)
    }
}

const crate = new ROCrate();
crate.index();
const root = crate.getRootDataset();
if (program.additionalContext) {
    const additionalContext = JSON.parse(fs.readFileSync(program.additionalContext));
    const vocab = additionalContext["@vocab"];
    delete additionalContext["@vocab"];
    const newContext = {};
    for (key of Object.keys(additionalContext)) {
        const id = `${vocab}${key}`
    
        const additionalProp = {
            "@id": id,
            "@type": "rdf:Property",
            "rdfs:label": key,  
            "rdf:comment": additionalContext[key]["description"],
        }
        crate.getGraph().push(additionalProp)
        newContext[key] = id;

    }
    crate.json_ld["@context"].push(newContext);
}
root.name = "Convictions Sample Data DEMO"
root.hasPart = [{
    "@id": "https://prov.vic.gov.au/explore-collection/explore-topic/justice-crime-and-law/register-male-and-female-prisoners-1855-1947"
  }];
root.hasMember = [];
crate.addItem({
    "@id": "https://prov.vic.gov.au/explore-collection/explore-topic/justice-crime-and-law/register-male-and-female-prisoners-1855-1947",
    "@type": "ArchiveOrganization",
    "name": "Register of Male and Female Prisoners (1855-1947). Public Record Office of Victoria."
  });

function getPlace(name) {
    if (!places[name]) {
        places[name] = {
            "@id": `#place_${name}`,
            "@type": "Place",
            "name": name
        }
        crate.getGraph().push(places[name]);

    }
    return places[name];
}

main(file, cratescript);

function findFiles(dataDir, files) {
    for (let f of fs.readdirSync(dataDir)) {
        var p = path.join(dataDir, f)
        stats = fs.lstatSync(p);
       if (stats.isDirectory()) {
            findFiles(p, files)
        }
        files[f.replace(/\..*$/, "")] = p;
        files[f.replace(/_0+/g, "_").replace(/\..*$/, "")] = p;
        
        
    }
}

async function main(file, cratescript) {
    if (program.places) {
        loadPlaces(program.places)
    }
    const destDir = path.dirname(file);
    const dataDir = program.data;
    var fileMap = {};
    if (dataDir) {
        findFiles(dataDir, fileMap);
    }
   //console.log(fileMap);
  
    var workbook = XLSX.readFile(file);
    const data = XLSX.utils.sheet_to_json(workbook.Sheets["Sheet1"]);
    //console.log(data);
    var count = 0;
    for (let row of data) {
        //console.log(Object.keys(row));
        const names = row["Name (and aliases)"]
                          .split(/;\s*/)
        //console.log(names);
        var birthplace = getPlace(row.Birthplace);
        const mainName = names.shift();
        var person = {
            "@id": `#person${row["Image ID"].padStart(22, "_")}`,
            "@type": "Person",
            "name": mainName,
            "alternateName": names.map((n)=>{return n.replace(/^./,"").replace(/.$/, "")}),
            "birthDate": row["Birth year"].toString(),
            "birthPlace": {"@id": birthplace["@id"]},
            "religion": row["Religion"],
            "education": row["Education"],
            "conviction": []
                    };


        for (let c of row["Conviction History"].split(";")) {
            const conv = c.split(/,\s*/);
           
            convictionPlace = getPlace(conv[3]);
            //6 MONTHS, 2-NOV-1904, VAGRANCY, MELBOURNE PETTY SESSIONS;
            if(conv[1]){
                const date = conv[1].toString();
                const offenceName = conv[2] || "Unspecified Offence";
                offenceID = `#offence_${offenceName.replace(/\W/g, "_")}`;
                if (!crate.getItem(offenceID)) {
                    crate.addItem ({
                        "@type" : "Offence",
                        "@id": offenceID,
                        "name"  : offenceName
                    });
                }

                var action = {
                    "@id": `#conviction_${count++}`,
                    "@type": ["Sentence"],
                    "object": {"@id": person["@id"]},
                    "name": `${date} ${mainName}: ${conv[0]} ${conv[2]} ${convictionPlace.name}`,
                    "sentence" : conv[0],
                    "offence": {"@id": offenceID},
                    "startTime": date,
                    "location": {"@id": convictionPlace["@id"]}
                }
                crate.addItem(action);
                person.conviction.push({"@id": action["@id"]});
            }
        }
        var filename = row["Image ID"];
        if (fileMap[filename]) {
            fileNameWithExt = path.basename(fileMap[filename]);
            const targetPath = path.join(destDir, fileNameWithExt);
            if (!fs.existsSync(targetPath)){
             console.log("Copying", targetPath);
             fs.copyFileSync(fileMap[filename], targetPath);
             }
             filename = fileNameWithExt;
        } else {
            console.log("Not found", filename);
        }
        var file = {
            "name": `${mainName}: record`,
            "@id": `${filename}`,
            "@type": ["File", "ArchiveComponent"],
            "holdingArchive": {"@id": "https://prov.vic.gov.au/explore-collection/explore-topic/justice-crime-and-law/register-male-and-female-prisoners-1855-1947"},
            "about": {"@id": person["@id"]}
        };
        person.hasFile = {"@id": file["@id"]}
        crate.addItem(person);
        crate.addItem(file);
        //root.hasMember.push({"@id": person["@id"]});

        
    }

    fs.writeFileSync(path.join(destDir, "ro-crate-metadata.json"), JSON.stringify(crate.json_ld, null, 2));
    const preview = await new Preview(crate);
    const f = new HtmlFile(preview);
    const html = path.join(destDir, "ro-crate-preview.html");
    fs.writeFileSync(html, await f.render(cratescript));
    //console.log("Conviction", conviction);
}



